#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Drawing Reflections" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

import sys

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from PIL import Image


FLOOR_SIZE = 20.
BOX_SIZE = 7
BOX_HEIGHT = BOX_SIZE


class myGLCanvas(object):
    def __init__(self):
        """
        Création du canevas recevant la modélisation 3D
        """
        self._angle = 30.

        glutInit(sys.argv)
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL)
        glutInitWindowSize(400, 400)
        glutCreateWindow(b"Drawing Reflections - videotutorialsrock.com")

        self.initRendering()
        self.loadTexture()
 
        glutTimerFunc(25, self.update, 0)
        
        glutDisplayFunc(self.OnPaint)
        glutReshapeFunc(self.OnResize)
        glutKeyboardFunc(self.handleKeypress)

        glutMainLoop()
        return 0
        
       
    def initRendering(self):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_NORMALIZE)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)


    def drawCube(self, textureId, angle):
        glDisable(GL_TEXTURE_2D)

        glPushMatrix()
        glRotatef(-angle, 1.0, 1.0, 0.0)

        glBegin(GL_QUADS)
        
        glColor3f(1.0, 1.0, 0.0)
        glNormal3f(0.0, 1.0, 0.0)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)

        glColor3f(1.0, 0.0, 1.0)
        glNormal3f(0.0, -1.0, 0.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)

        glNormal3f(-1.0, 0.0, 0.0)
        glColor3f(0.0, 1.0, 1.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glColor3f(0.0, 0.0, 1.0)        
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)

        glNormal3f(1.0, 0.0, 0.0)
        glColor3f(1.0, 0.0, 0.0)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glColor3f(0.0, 1.0, 0.0)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        
        glEnd()

        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glColor3f(1.0, 1.0, 1.0)

        glBegin(GL_QUADS)
        
        glNormal3f(0.0, 0.0, 1.0)
        glTexCoord2f(0.0, 0.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glTexCoord2f(1.0, 0.0)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glTexCoord2f(1.0, 1.0)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glTexCoord2f(0.0, 1.0)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)

        glNormal3f(0.0, 0.0, -1.0)
        glTexCoord2f(0.0, 0.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glTexCoord2f(1.0, 0.0)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glTexCoord2f(1.0, 1.0)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glTexCoord2f(0.0, 1.0)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        
        glEnd()
        
        glPopMatrix()

        
    def drawFloor(self, textureId):
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        
        glBegin(GL_QUADS)
        
        glNormal3f(0, 1, 0)
        glTexCoord2f(0, 0)
        glVertex3f(-FLOOR_SIZE/2, 0, FLOOR_SIZE/2)
        glTexCoord2f(0, 1)
        glVertex3f(-FLOOR_SIZE/2, 0, -FLOOR_SIZE/2)
        glTexCoord2f(1, 1)
        glVertex3f(FLOOR_SIZE/2, 0, -FLOOR_SIZE/2)
        glTexCoord2f(1, 0)
        glVertex3f(FLOOR_SIZE/2, 0, FLOOR_SIZE/2)
        glEnd()


    def OnPaint(self):
        glClear(GL_COLOR_BUFFER_BIT | \
                GL_DEPTH_BUFFER_BIT | \
                GL_STENCIL_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        glTranslatef(0.0, 0.0, -40.0)
        glRotatef(30, 1, 0, 0)

        ambientLight = [0.3, 0.3, 0.3, 1.0]
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight)
        
        directedLight = [0.7, 0.7, 0.7, 1.0]
        directedLightPos = [-2*BOX_SIZE, 4*BOX_SIZE, 1.0]
        glLightfv(GL_LIGHT0, GL_DIFFUSE, directedLight)
        glLightfv(GL_LIGHT0, GL_POSITION, directedLightPos)
        
        glPushMatrix()
        glTranslatef(0, BOX_HEIGHT, 0)
        self.drawCube(self.textureId, self._angle)
        glPopMatrix()
        
        glEnable(GL_STENCIL_TEST)
        glColorMask(0, 0, 0, 0)
        glDisable(GL_DEPTH_TEST)
        glStencilFunc(GL_ALWAYS, 1, 1)
        
        glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE)
        
        self.drawFloor(self.textureId)
        
        glColorMask(1, 1, 1, 1)
        glEnable(GL_DEPTH_TEST)
        
        glStencilFunc(GL_EQUAL, 1, 1)
        glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP)
        
        glPushMatrix()
        glScalef(1, -1, 1)
        glTranslatef(0, BOX_HEIGHT, 0)
        self.drawCube(self.textureId, self._angle)
        glPopMatrix()
        
        glDisable(GL_STENCIL_TEST)
        
        glEnable(GL_BLEND)
        glColor4f(1, 1, 1, 0.7)
        self.drawFloor(self.textureId)
        glDisable(GL_BLEND)

        glutSwapBuffers()
        
          
    def OnResize(self, w, h):
        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)
        
    
    def update(self, value):
        self._angle += 1.
        if self._angle > 360. : self._angle -= 360.
        
        glutPostRedisplay()
        glutTimerFunc(25, self.update, 0)
        

    def handleKeypress(self, key, x, y):
        if ord(key) == 27 : exit()


    def loadTexture(self):
        self.textureId = glGenTextures(1)
        
        try:
            image = Image.open("vtr.bmp")
        except:
            print ("Chargement de la texture impossible : fichier introuvable")
            return

        w, h = image.size
        pixels = image.transpose(Image.FLIP_TOP_BOTTOM).tobytes()
        
        del image
        
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexImage2D(GL_TEXTURE_2D, \
                     0, \
                     GL_RGB, \
                     w, h, \
                     0, \
                     GL_RGB, \
                     GL_UNSIGNED_BYTE, \
                     pixels)
        
 
if __name__ == '__main__':
    canvas = myGLCanvas()
